@@ -1,19 +1,23 @@
-from matplotlib.axes import Axes
-import matplotlib.pyplot as plt
-from pathlib import Path
-from typing import Optional, Union, Dict, Callable, List
+from pandas import DataFrame
 
+from compound_types.arrays import ArrayLike
+from compound_types.built_ins import FloatOrFloatIterable, StrOrStrIterable, \
+    DictOrDictIterable
+import matplotlib.pyplot as plt
+from matplotlib.axes import Axes
 from matplotlib.collections import PathCollection
 from matplotlib.font_manager import FontProperties
-from numpy.core.records import ndarray
-
+from matplotlib.patches import Rectangle
 from mpl_format.axes.axis_formatter import AxisFormatter
 from mpl_format.axes.axis_utils import new_axes
-from mpl_format.compound_types import FontSize, Color, LegendLocation, FloatOrFloats, ArrayLike
+from mpl_format.compound_types import FontSize, Color, LegendLocation
 from mpl_format.io_utils import save_plot
 from mpl_format.legend.legend_formatter import LegendFormatter
+from mpl_format.patches.patch_list_formatter import PatchListFormatter
 from mpl_format.text.text_formatter import TextFormatter
 from mpl_format.text.text_utils import wrap_text
+from pathlib import Path
+from typing import Optional, Union, Dict, Callable, List, Tuple, Iterable
 
 
 class AxesFormatter(object):
@@ -30,8 +34,10 @@ class AxesFormatter(object):
         :param constrained_layout: Option for constrained_layout of new Axes.
         """
         if axes is None:
-            self._axes: Axes = new_axes(width=width, height=height,
-                                        constrained_layout=constrained_layout)
+            self._axes: Axes = new_axes(
+                width=width, height=height,
+                constrained_layout=constrained_layout
+            )
         else:
             self._axes: Axes = axes
         self._x_axis: AxisFormatter = AxisFormatter(self._axes.xaxis)
@@ -570,8 +576,9 @@ class AxesFormatter(object):
         )
         return self
 
-    def add_h_lines(self, y: FloatOrFloats,
-                    x_min: FloatOrFloats, x_max: FloatOrFloats,
+    def add_h_lines(self, y: FloatOrFloatIterable,
+                    x_min: FloatOrFloatIterable,
+                    x_max: FloatOrFloatIterable,
                     colors='k', line_styles: str = 'solid',
                     label: Optional[str] = '') -> 'AxesFormatter':
         """
@@ -589,8 +596,9 @@ class AxesFormatter(object):
                           label=label)
         return self
 
-    def add_v_lines(self, x: FloatOrFloats,
-                    y_min: FloatOrFloats, y_max: FloatOrFloats,
+    def add_v_lines(self, x: FloatOrFloatIterable,
+                    y_min: FloatOrFloatIterable,
+                    y_max: FloatOrFloatIterable,
                     colors='k', line_styles: str = 'solid',
                     label: Optional[str] = '') -> 'AxesFormatter':
         """
@@ -608,9 +616,10 @@ class AxesFormatter(object):
                           label=label)
         return self
 
-    def fill_between(self, x: ArrayLike,
-                     y1: Union[float, ArrayLike],
-                     y2: Union[float, ArrayLike],
+    def fill_between(self, x: Union[ArrayLike, str],
+                     y1: Union[float, ArrayLike, str],
+                     y2: Union[float, ArrayLike, str],
+                     data: Optional[DataFrame] = None,
                      where: Optional[ArrayLike] = None,
                      interpolate: bool = False,
                      step: Optional[str] = None,
@@ -623,9 +632,10 @@ class AxesFormatter(object):
         """
         Make filled polygons between two curves.
 
-        :param x: An N-length array of the x data.
-        :param y1: An N-length array (or scalar) of the y data.
-        :param y2: An N-length array (or scalar) of the y data.
+        :param x: N-length array of, or name of column with the x data.
+        :param y1: N-length array, scalar, or name of column with the y1 data.
+        :param y2: N-length array, scalar, or name of column with the y2 data.
+        :param data: Optional DataFrame with x, y1 and y2 columns.
         :param where: If None, default to fill between everywhere. If not None,
                       it is an N-length numpy boolean array and the fill will
                       only happen over the regions where where==True.
@@ -642,6 +652,15 @@ class AxesFormatter(object):
         :param edge_color: matplotlib color spec or sequence of specs
         :param face_color: matplotlib color spec or sequence of specs
         """
+        # get arrays from DataFrame
+        if data is not None:
+            if isinstance(x, str):
+                x = data[x]
+            if isinstance(y1, str):
+                y1 = data[y1]
+            if isinstance(y2, str):
+                y2 = data[y2]
+        # convert args to matplotlib names
         kwargs = {}
         for arg, mpl_arg in zip(
             [color, alpha, line_style, line_width, edge_color, face_color],
@@ -650,7 +669,7 @@ class AxesFormatter(object):
         ):
             if arg is not None:
                 kwargs[mpl_arg] = arg
-
+        # call matplotlib method
         self._axes.fill_between(
             x=x, y1=y1, y2=y2,
             where=where, interpolate=interpolate,
@@ -668,12 +687,32 @@ class AxesFormatter(object):
         self._axes.set_facecolor(color)
         return self
 
+    def get_frame_color(self) -> Union[Color, List[Color]]:
+        """
+        Return the color of the frame if all edges are the same color,
+        otherwise a list of the top, bottom, left and right colors.
+        """
+        colors = self.get_frame_colors()
+        if len(set(colors)) == 1:
+            return colors[0]
+        else:
+            return self.get_frame_colors()
+
     def set_frame_color(self, color: Color) -> 'AxesFormatter':
 
-        for pos in ['top', 'bottom', 'right', 'left']:
+        for pos in ['top', 'bottom', 'left', 'right']:
             self._axes.spines[pos].set_edgecolor(color)
         return self
 
+    def get_frame_colors(self) -> List[Color]:
+        """
+        Return the colors of the top, bottom, left and right edges of the Axes.
+        """
+        return [
+            self._axes.spines[pos].get_edgecolor()
+            for pos in ['top', 'bottom', 'left', 'right']
+        ]
+
     def set_tick_color(self, color: Color) -> 'AxesFormatter':
 
         self.x_axis.set_tick_color(color=color)
@@ -682,6 +721,58 @@ class AxesFormatter(object):
 
     # endregion
 
+    # region shapes
+
+    @property
+    def rectangles(self) -> PatchListFormatter:
+        """
+        Return a list of the Rectangles on the axes.
+        """
+        return PatchListFormatter([
+            r for r in self._axes.get_children()
+            if isinstance(r, Rectangle)
+        ])
+
+    def add_text(
+            self, x: FloatOrFloatIterable,
+            y: FloatOrFloatIterable,
+            text: StrOrStrIterable,
+            font_dict: Optional[DictOrDictIterable] = None,
+            **kwargs
+    ) -> 'AxesFormatter':
+        """
+        Add a single or multiple text blocks to the plot.
+
+        :param x: x-coordinate(s) of the text.
+        :param y: y-coordinate(s) of the text.
+        :param text: String or iterable of text strings to add.
+        :param font_dict: A dictionary to override the default text properties.
+                          If font_dict is None, the defaults are determined by
+                          your rc parameters.
+        :param kwargs: Other miscellaneous Text parameters.
+        """
+        if isinstance(x, Iterable) or isinstance(y, Iterable):
+            if not isinstance(x, Iterable):
+                x = [x] * len(y)
+            elif not isinstance(y, Iterable):
+                y = [y] * len(x)
+        else:
+            x = [x]
+            y = [y]
+            if isinstance(text, str):
+                text = [text] * len(x)
+            if type(font_dict) in (type(None), dict):
+                font_dict = [font_dict] * len(x)
+
+        for x_i, y_i, text_i, font_dict_i in zip(x, y, text, font_dict):
+            self._axes.text(
+                x=x_i, y=y_i, s=text_i,
+                fontdict=font_dict_i, **kwargs
+            )
+        return self
+
+    # endregion
+
     def set_title_font_family(self, font_name: str) -> 'AxesFormatter':
         """
         Set the font family for the Axes title.
@@ -691,6 +782,20 @@ class AxesFormatter(object):
         self.title.set_font_family(font_name)
         return self
 
+    # region limits
+
+    def get_x_lim(self) -> Tuple[float, float]:
+        """
+        Return the x-axis view limits.
+        """
+        return self._axes.get_xlim()
+
+    def get_y_lim(self) -> Tuple[float, float]:
+        """
+        Return the y-axis view limits.
+        """
+        return self._axes.get_ylim()
+
     def set_x_lim(self, left: Optional[float] = None,
                   right: Optional[float] = None) -> 'AxesFormatter':
         """
@@ -705,7 +810,7 @@ class AxesFormatter(object):
     def set_y_lim(self, bottom: Optional[float] = None,
                   top: Optional[float] = None) -> 'AxesFormatter':
         """
-        Set the limits of the x-axis.
+        Set the limits of the y-axis.
 
         :param bottom: Lower limit.
         :param top: Upper limit.
@@ -713,6 +818,60 @@ class AxesFormatter(object):
         self._axes.set_ylim(bottom=bottom, top=top)
         return self
 
+    def get_x_min(self) -> float:
+        """
+        Return the x-axis lower view limit.
+        """
+        return self.get_x_lim()[0]
+
+    def get_x_max(self) -> float:
+        """
+        Return the x-axis upper view limit.
+        """
+        return self.get_x_lim()[1]
+
+    def set_x_min(self, left: float = None) -> 'AxesFormatter':
+        """
+        Set the x-axis lower view limit.
+        """
+        self.set_x_lim(left, None)
+        return self
+
+    def set_x_max(self, right: float = None) -> 'AxesFormatter':
+        """
+        Set the x-axis upper view limit.
+        """
+        self.set_x_lim(None, right)
+        return self
+
+    def get_y_min(self) -> float:
+        """
+        Return the y-axis lower view limit.
+        """
+        return self.get_y_lim()[0]
+
+    def get_y_max(self) -> float:
+        """
+        Return the y-axis upper view limit.
+        """
+        return self.get_y_lim()[1]
+
+    def set_y_min(self, left: float = None) -> 'AxesFormatter':
+        """
+        Set the y-axis lower view limit.
+        """
+        self.set_y_lim(left, None)
+        return self
+
+    def set_y_max(self, right: float = None) -> 'AxesFormatter':
+        """
+        Set the y-axis upper view limit.
+        """
+        self.set_y_lim(None, right)
+        return self
+
+    # endregion
+
     def grid(self, value: bool,
              which: str = 'major',
              axis: str = 'both') -> 'AxesFormatter':
